@@ -1,7 +1,7 @@
 use strictures;
 
 package WebService::GoogleAPI::Client::UserAgent;
-$WebService::GoogleAPI::Client::UserAgent::VERSION = '0.20';
+$WebService::GoogleAPI::Client::UserAgent::VERSION = '0.21';
 
 # ABSTRACT: User Agent wrapper for working with Google APIs
 
@@ -15,7 +15,7 @@ use WebService::GoogleAPI::Client::AuthStorage;
 use Mojo::UserAgent;
 use Data::Dump qw/pp/;    # for dev debug
 
-use Carp;
+use Carp qw/croak carp cluck/;
 
 has 'do_autorefresh'                => ( is => 'rw', default => 1 );    # if 1 storage must be configured
 has 'auto_update_tokens_in_storage' => ( is => 'rw', default => 1 );
@@ -51,14 +51,14 @@ sub header_with_bearer_auth_token
 
   $headers->{ 'Accept-Encoding' } = 'gzip';
 
-  # carp "header_with_bearer_auth_token: ".$self->access_token;
+  # cluck "header_with_bearer_auth_token: ".$self->access_token;
   if ( $self->access_token )
   {
     $headers->{ 'Authorization' } = 'Bearer ' . $self->access_token;
   }
   else
   {
-    carp 'No access_token, can\'t build Auth header';
+    cluck 'No access_token, can\'t build Auth header';
   }
   return $headers;
 }
@@ -72,15 +72,15 @@ sub build_http_transaction
   $params->{ httpMethod } = $params->{ method } if defined $params->{ method };
   $params->{ httpMethod } = '' unless defined $params->{ httpMethod };
 
-  my $http_method   = uc( $params->{ httpMethod } ) || 'GET';                                                          # uppercase ?
+  my $http_method   = uc( $params->{ httpMethod } ) || 'GET';                                                           # uppercase ?
   my $optional_data = $params->{ options }          || '';
-  my $path          = $params->{ path }             || carp( 'path parameter required for build_http_transaction' );
-  my $no_auth       = $params->{ no_auth }          || 0;                                                              ## default to including auth header - ie not setting no_auth
+  my $path          = $params->{ path }             || cluck( 'path parameter required for build_http_transaction' );
+  my $no_auth       = $params->{ no_auth }          || 0;                                                               ## default to including auth header - ie not setting no_auth
   my $headers       = $params->{ headers }          || {};
 
-  carp 'Attention! You are using POST, but no payload specified' if ( ( $http_method eq 'POST' ) && !defined $optional_data );
-  carp "build_http_transaction:: $http_method $path " if $self->debug;
-  carp "$http_method Not a SUPPORTED HTTP method parameter specified to build_http_transaction" . pp $params unless $http_method =~ /^GET|PATH|PUT|POST|PATCH|DELETE$/ixm;
+  cluck 'Attention! You are using POST, but no payload specified' if ( ( $http_method eq 'POST' ) && !defined $optional_data );
+  cluck "build_http_transaction:: $http_method $path " if ( $self->debug > 11 );
+  cluck "$http_method Not a SUPPORTED HTTP method parameter specified to build_http_transaction" . pp $params unless $http_method =~ /^GET|PATH|PUT|POST|PATCH|DELETE$/ixm;
 
   ## NB - headers not passed if no_auth
   $headers = $self->header_with_bearer_auth_token( $headers ) unless $no_auth;
@@ -124,12 +124,15 @@ sub validated_api_query
 
   if ( ref( $params ) eq '' )    ## assume is a GET for the URI at $params
   {
-    carp( "transcribing $params to a hashref for validated_api_query" ) if $self->debug;
+    cluck( "transcribing $params to a hashref for validated_api_query" ) if $self->debug;
     my $val = $params;
     $params = { path => $val, method => 'get', options => {}, };
   }
 
-  my $res = $self->start( $self->build_http_transaction( $params ) )->res;
+  my $tx = $self->build_http_transaction( $params );
+
+  cluck( "$params->{method} $params->{path}" ) if $self->debug;
+  my $res = $self->start( $tx )->res;
 
   ## TODO: HANDLE TIMEOUTS AND OTHER ERRORS IF THEY WEREN'T HANDLED BY build_http_transaction
 
@@ -141,13 +144,13 @@ sub validated_api_query
     if ( $res->code == 401 )    ## redundant - was there something else in mind ?
     {
       croak "No user specified, so cant find refresh token and update access_token" unless $self->user;
-      carp "401 response - access_token was expired. Attemptimg to update it automatically ..." if $self->debug;
+      cluck "401 response - access_token was expired. Attemptimg to update it automatically ..." if ( $self->debug > 11 );
 
-      # carp "Seems like access_token was expired. Attemptimg update it automatically ..." if $self->debug;
+      # cluck "Seems like access_token was expired. Attemptimg update it automatically ..." if $self->debug;
 
       my $cred      = $self->auth_storage->get_credentials_for_refresh( $self->user );    # get client_id, client_secret and refresh_token
       my $new_token = $self->refresh_access_token( $cred )->{ access_token };             # here also {id_token} etc
-      carp "validated_api_query() Got a new token: " . $new_token if $self->debug;
+      cluck "validated_api_query() Got a new token: " . $new_token if ( $self->debug > 11 );
       $self->access_token( $new_token );
 
       if ( $self->auto_update_tokens_in_storage )
@@ -156,18 +159,19 @@ sub validated_api_query
       }
 
       #$tx  = $self->build_http_transaction( $params );
+
       $res = $self->start( $self->build_http_transaction( $params ) )->res;               # Mojo::Message::Response
     }
   }
   elsif ( $res->code == 403 )
   {
-    carp( 'Unexpected permission denied 403 error ' );
+    cluck( 'Unexpected permission denied 403 error ' );
     return $res;
   }
   return $res if $res->code == 200;
   return $res if $res->code == 204;                                                       ## NO CONTENT - INDICATES OK FOR DELETE ETC
   return $res if $res->code == 400;                                                       ## general failure
-  carp( "unhandled validated_api_query response code " . $res->code );
+  cluck( "unhandled validated_api_query response code " . $res->code );
   return $res;
 }
 
@@ -191,7 +195,7 @@ sub refresh_access_token
     croak "Not enough credentials to refresh access_token. Check that you provided client_id, client_secret and refresh_token";
   }
 
-  carp "refresh_access_token:: Attempt to refresh access_token " if $self->debug;
+  cluck "refresh_access_token:: Attempt to refresh access_token " if ( $self->debug > 11 );
   $credentials->{ grant_type } = 'refresh_token';
   return $self->post( 'https://www.googleapis.com/oauth2/v4/token' => form => $credentials )->res->json || croak( 'refresh_access_token failed' );    # tokens
 }
@@ -210,7 +214,7 @@ WebService::GoogleAPI::Client::UserAgent - User Agent wrapper for working with G
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head2 C<header_with_bearer_auth_token>
 
@@ -230,7 +234,7 @@ version 0.20
 
 Google API HTTP 'method' request to API End Point at 'path' with optional parameters described by 'options'
 
-By 'validated' I mean that no checks are performed against dsivoery data structures and no interpolation is performed.
+By 'validated' I mean that no checks are performed against discovery data structures and no interpolation is performed.
 The pre-processing functionality for the library is expected to be completed by the 'Client' class before passing
 the cleaner, sanitised and validated request to the agent here for submitting.
 
